% diox_levels    field-dependent energy levels of dioxygen
%
%    diox_levels(FieldRange,nPoints,Nmax,Ndisplay)
%    Energy = ...
%
%    This function computes the energy levels of dioxygen in its
%    ground state as a function of the external magnetic field strength.
%
%    FieldRange ...   [minField maxField], in Tesla
%    nPoints    ...   number of points in the field range
%    Nmax       ...   maximum quantum number N
%    Ndisplay   ...   N for the levels to be displayed
%                     (if not given, nothing is plotted)
%
%    Energy     ...   2D array with all energies for a given field
%                     along columns. In MHz.
%
%    Example:
%
%        diox_levels([0 1.3],1001,7,3);
%        diox_levels([0 1.3],501,7,[1 3]);

function Energy = diox_levels(Range,nPoints,Nmax,Ndisplay)

% Construction Hamiltonian
%-------------------------------------------------------------
[J,N,M] = diox_basis(Nmax);
[H0,H1] = diox_hamiltonian(Nmax);
nBasis = length(J);

% Computing energy levels
%-------------------------------------------------------------
Field = linspace(Range(1),Range(2),nPoints);

Energy = zeros(nBasis,nPoints);
for iField = 1:nPoints
  Hamiltonian = H0 + H1*Field(iField);
  Energy(:,iField) = diox_eig(Hamiltonian,M);
end

% Plot energy level diagram
%-------------------------------------------------------------
if (nargin<4), Ndisplay = []; end
if (Ndisplay>Nmax)
  error('Cannot display levels that have not been computed!');
end

if ~isempty(Ndisplay)
  [J,N,m] = diox_basis(Nmax);
  idxShow = ismember(N,Ndisplay);
  plot(Field,Energy(idxShow,:),'k');
  axis tight
  xlabel('magnetic field [T]');
  ylabel('energy [GHz]');
  title(['Energy levels for ' sprintf('N=%d ',Ndisplay)]);
end
