% diox_matrices   Operator matrices for the O2 Hamiltonian
%
%    [N2,NS,SzS2,Sz,Sx] = diox_matrices(JJ,NN,mm)
%
%    Input: basis
%      JJ ... vector of J
%      NN ... vector of N
%      mm ... vector of m
%
%    Output: matrices
%      N2   ... rotation term N*N
%      NS   ... spin-rotation term N*S
%      SzS2 ... spin-spin term  3*Sz - S^2
%      Sz   ... space-fixed z component of S
%      Sx   ... space-fixed x component of S

function [N2,NS,SzS2,Sz,Sx] = diox_matrices(JJ,NN,mm)

nb = numel(JJ);

% Matrix elements of simple operators
%---------------------------------------------------
% diagonal in J, N, m
N2 = diag(NN.*(NN+1));
NS = diag(JJ.*(JJ+1)-NN.*(NN+1)-2)/2;

% Matrix elements of 3Sz'^2-S^2 (molecule fixed z')
%---------------------------------------------------
% diagonal in J and m
% Delta N = -2,0,+2
SzS2 = zeros(nb);
for b1 = 1:nb
  for b2 = 1:nb
    if (JJ(b1)~=JJ(b2)) || (mm(b1)~=mm(b2)), continue; end
    J = JJ(b1); N = NN(b1);
    val = 0;
    switch NN(b2)
      case N
        switch J
          case N+1, val = -N/(2*N+3);
          case N,   val = 1;
          case N-1, val = -(N+1)/(2*N-1);
        end
      case N+2
        if (J==N+1), val = 3/(2*N+3)*sqrt((N+1)*(N+2)); end
      case N-2
        if (J==N-1), val = 3/(2*N-1)*sqrt(N*(N-1)); end
    end
    SzS2(b1,b2) = val;
  end
end

% Matrix elements of Sz (space-fixed z)
%---------------------------------------------------
% diagonal in N and m
Sz = zeros(nb);
for b1 = 1:nb
  for b2 = 1:nb
    if (NN(b1)~=NN(b2)) || (mm(b1)~=mm(b2)), continue; end
    m = mm(b1); N = NN(b1);
    val = 0;
    if JJ(b2)==JJ(b1)
      switch JJ(b1)
        case N+1, val =  m/(N+1);
        case N,   val =  m/N/(N+1);
        case N-1, val = -m/N;
      end
    elseif (JJ(b1)+JJ(b2)==2*N+1)
      val = -N/(N+1)*sqrt(((N+1)^2-m^2)/N/(2*N+1)); 
    elseif (JJ(b1)+JJ(b2)==2*N-1)
      val = -(N+1)/N*sqrt((N^2-m^2)/(N+1)/(2*N+1));
    end
    Sz(b1,b2) = val;
  end
end


% Matrix elements of S+ (space-fixed x+iy)
%-----------------------------------------------------------
% diagonal in N
% one m diagonal <m+1,m>, Delta m = +1
Sp = zeros(nb);
for b1 = 1:nb
  for b2 = 1:nb
    if (NN(b1)~=NN(b2)) || (mm(b1)~=mm(b2)+1), continue; end
    N = NN(b1); m = mm(b2);
    val = 0;
    switch JJ(b2)
      case N+1
        switch JJ(b1)
          case N+1, val =  sqrt((N+m+2)*(N-m+1))/(N+1);
          case N,   val = -sqrt((N-m)*(N-m+1)*N/(2*N+1))/(N+1);
        end
      case N
        switch JJ(b1)
          case N+1, val =  sqrt((N+m+1)*(N-m+2)*N/(2*N+1))/(N+1);
          case N,   val =  sqrt((N+m+1)*(N-m))/N/(N+1);
          case N-1, val = -sqrt((N-m)*(N-m-1)*(N+1)/(2*N+1))/N;
        end
      case N-1
        switch JJ(b1)
          case N,   val =  sqrt((N+m)*(N+m+1)*(N+1)/(2*N+1))/N;
          case N-1, val = -sqrt((N+m)*(N-m-1))/N;
        end
    end
    Sp(b1,b2) = val;
  end
end
Sx = (Sp + Sp')/2;
