% diox_resonances     Computes all EPR resonance fields for dioxygen
%
%    diox_resonances(FieldRange,Nmax,mwFreq,Temp,nPoints,Ndisplay)
%    Reson = ...
%
%    FieldRange ... [minField, maxField], in Tesla
%    Nmax       ... maximum quantum number N
%    mwFreq     ... microwave frequency, in GHz
%
%    Temp       ... temperature, in Kelvin (optional, default 295 K)
%    nPoints    ... number of field points (optional, default 1024)
%    Ndisplay   ... N of the levels to display (if not given, levels
%                   and resonances are not plotted)
% 
%    Reson ... array with resonances
%              first column: quantum number N
%              second column: resonance field, mT
%              third column: transition intensity, a.u.
%
%    Example:
%
%        diox_resonances([0 1],11,9.8,293,1001,1);

function Resonances = diox_resonances(FieldRange,Nmax,mwFreq,nPoints,Temp,Ndisplay)

if (nargin<4), nPoints = 1024; end
if (nargin<5), Temp = 293; end
if (nargin<6), Ndisplay = []; end

Pars.Threshold = 5e-3;

kB = 1.3806505e-23;       % Boltzmann constant [J/K]
h = 6.6260693e-34;        % Planck constant [J s]

mwFreq = mwFreq*1e3; % GHz -> MHz

Field = linspace(FieldRange(1),FieldRange(2),nPoints);
dB = Field(2)-Field(1);

% Getting basis and Hamiltonian matrices
%-------------------------------------------------------------
[J,N,M] = diox_basis(Nmax);
nBasis = numel(J);
[Ham0,Ham1,Sx] = diox_hamiltonian(Nmax);


% Computing energy levels and determining resonance positions
%--------------------------------------------------------------
% running over all intervals: Field(iF-1)...Field(iF)
r = 1;
E2 = eig(Ham0+Field(1)*Ham1);
E(1,:) = E2.';
for iF = 2:nPoints

  % Left border
  E1 = E2;

  % Diagonalize Hamiltonian on right border
  % (energies returned are sorted in ascending order)
  [Vec,E2] = diox_eig(Ham0 + Field(iF)*Ham1,M);
  E(iF,:) = E2.';

  for ilo = 1:nBasis-1
    for ihi = ilo+1:nBasis
      if N(ihi)>N(ilo), break; end

      % Energy differences on left and right
      dE1 = E1(ihi)-E1(ilo);
      dE2 = E2(ihi)-E2(ilo);

      % Position of potential resonance, must be between
      % 0 and 1, if there is a resonance in the current interval.
      res = (dE1-mwFreq)/(dE1-dE2);

      if (res>0)&&(res<1)

        % Interpolate resonance field
        resField = Field(iF-1)*(1-res) + Field(iF)*res;

        % Compute population difference of the two levels
        Populations = exp(-h*E2*1e6/kB/Temp);
        Populations = Populations/sum(Populations);
        PopDiff = Populations(ilo)-Populations(ihi);

        % Compute Sx matrix element square between the two levels
        Trans = abs(Vec(:,ihi)'*Sx*Vec(:,ilo))^2;

        % Derivative of energy difference with respect to field
        dEdB = abs(dE2-dE1)/dB;

        Resonances(r,:) = [N(ilo), resField,  Trans*PopDiff*dEdB];
        % Interpolated energy of the two levels involved in the transition, for plotting
        Energies(r,:) = [E1(ilo)*(1-res)+E2(ilo)*res,E1(ihi)*(1-res)+E2(ihi)*res];
        r = r + 1;
      end
      
    end
  end
  
end

nResonances = r-1;

% Remove low-intensity resonances
RemoveLowInt = 1;
if (RemoveLowInt) & (nResonances>0)
  Intens = Resonances(:,3);
  lowint = Intens < Pars.Threshold*max(Intens);
  Resonances(lowint,:) = [];
  Energies(lowint,:) = [];
end

if ~isempty(Ndisplay)
  idx = ismember(N,Ndisplay);
  plot(Field,E(:,idx)/1e3,'k');
  if (nResonances>0)
    show = ismember(Resonances(:,1),Ndisplay);
    x = [1;1]*Resonances(show,2).';
    y = Energies(show,:).'/1e3;
    line(x,y,'Color','r','LineWidth',1);
  end
  axis tight;
  xlabel('magnetic field [T]');
  ylabel('energy [GHz]');
end
