% diox_sim      Simulates the EPR spectrum of dioxygen
%
%    diox_sim(FieldRange,Nmax,mwFreq,Temp)
%    [B,spc] = ...
%
%    Input:
%      FieldRange ...  [minField maxField], in T
%      Nmax       ...  maximum rotational quantum number N
%      mwFreq     ...  microwave frequency, in GHz
%
%      Temp       ...  temperature (optional, default 293 K)
%    Output:
%      B          ...  magnetic field range, in T
%      spc        ..   EPR spectrum
%
%    If no output arguments are given, the spectrum is plotted.
%    If output arguments B and spc are requested, the spectrum
%    is not plotted.
%
%    Example:
%
%        diox_sim([0.54 0.78],15,9.5295);

function [B,spc] = diox_sim(FieldRange,Nmax,mwFreq,Temp)

if (nargin<4), Temp = 293; end

Pars.lw = 0.0005;  % Tesla
Pars.nPoints = 10000; % spectral points
nResPoints = 501; % point used in resonance field computations

% Compute resonance line positions and intensities
%--------------------------------------------------------------
Resonances = diox_resonances(FieldRange,Nmax,mwFreq,Temp,nResPoints);

% Construct spectrum
%--------------------------------------------------------------
% skip resonances for N=Nmax, since these are wrong due to
% truncation errors.
spc = zeros((Nmax-1)/2,Pars.nPoints);
B = linspace(FieldRange(1),FieldRange(2),Pars.nPoints);
singleN = [];
for r = 1:size(Resonances,1)
  thisN = Resonances(r,1);
  if thisN==Nmax, continue; end
  if ~isempty(singleN)
    if (thisN~=singleN), continue; end
  end
  iN = Resonances(r,1)/2+1/2;
  q = (B-Resonances(r,2))/Pars.lw;
  newLine = -q.*(1+4/3*q.^2).^-2;
  spc(iN,:) = spc(iN,:) + Resonances(r,3)*newLine;
end

% Normalize spectrum
%--------------------------------------------------------------
spc = spc/max(max(spc));

if (nargout==0)
  idx = 1;
  for n=1:2:Nmax-1
    str{idx} = sprintf('N = %d',n);
    idx = idx + 1;
  end
  plot(B,spc);
  axis tight
  legend(str{:});
  xlabel('magnetic field [T]');
  ylabel('intensity [a.u.]');
end
